package InterShipper;

use URI;
use LWP::Simple;
use HTML::Parser;

## globals
my $API_URL = "http://www.intershipper.com/Interface/Intershipper/XML/v2.0/HTTP.jsp";
my $VERSION = "2.0.0.0";
my $USERNAME = 'doug';
my $PASSWORD = 'password';
## end globals

sub set_credentials {		# call this once at beginning of program
	($USERNAME, $PASSWORD, $VERSION, $API_URL) = @_;
}

sub print_data {
	my $result = $_[0];
	require Data::Dumper;
	$Data::Dumper::Indent = 1;
	print Data::Dumper::Dumper($result);
}

sub get_menu_data {
	my $result = $_[0];
	my (%amount,%carrier_code);
	for my $package_id (keys %$result){
		for my $carrier_service (keys %{$result->{$package_id}}){
			$amount{$carrier_service} += $result->{$package_id}{$carrier_service}{amount};
			$carrier_code{$carrier_service} = $result->{$package_id}{$carrier_service}{carrier_name} . " " . $result->{$package_id}{$carrier_service}{service_name};		
		}
	}
	my @data = (\%amount,\%carrier_code);
	return (\@data);
}

sub compute_charges {		# pass this that huge param list
	my %params = @_;

	## patch up the multiple items
	for my $type (qw(Carriers Classes Packages Options)) {
		if ($params{$type}) {
			my $arrayref = delete $params{$type};
			$params{"Total$type"} = @$arrayref; # count
			for my $n (1..@$arrayref) {
				my $hashref = $arrayref->[$n-1];
				for (keys %$hashref) {
					$params{"$_$n"} = $hashref->{$_};
				}
			}
		}
	}

	my $uri = URI->new($API_URL);
	$uri->query_form(
		 Username => $USERNAME,
		 Password => $PASSWORD,
		 Version => $VERSION,
		 %params,
		);

	my $result = get($uri->as_string);
	my @state = ();
	my $error;

	my $package_id;
	my $boxID;
	my %quote;

	my %quotes;

	my $p = HTML::Parser->new(
		xml_mode => 1,
   		start_h => [ sub {
  			my ($tagname, $attr) = @_;
  			push @state, $tagname;
  			if ("@state" eq "shipment package") {
				$package_id = $attr->{id};
  			} 
  			elsif ("@state" eq "shipment package quote") {
				%quote = ( package_id => $package_id, id => $attr->{id} );
  			}
		}, "tagname, attr"],
   		text_h => [ sub {
  			my ($text) = @_;
  			if ("@state" eq "shipment error") {
				$error = $text;
  			} 
  			elsif ("@state" eq "shipment package boxID") {
				$boxID = $text;
  			} 
  			elsif ("@state" eq "shipment package quote carrier name") {
				$quote{carrier_name} = $text;
			} 
  			elsif ("@state" eq "shipment package quote carrier code") {
				$quote{carrier_code} = $text;
  			} 
  			elsif ("@state" eq "shipment package quote class name") {
				$quote{class_name} = $text;
  			} 
  			elsif ("@state" eq "shipment package quote class code") {
				$quote{class_code} = $text;
  			} 
  			elsif ("@state" eq "shipment package quote service name") {
				$quote{service_name} = $text;
  			} 
  			elsif ("@state" eq "shipment package quote service code") {
				$quote{service_code} = $text;
  			} 
  			elsif ("@state" eq "shipment package quote rate amount") {
				$quote{amount} = $text;
  			}
		}, "dtext"],
   		end_h => [ sub {
  			my ($tagname) = @_;
  			if ("@state" eq "shipment package quote") {
				my $key = "@quote{qw(carrier_code service_code)}";
				delete @quote{qw(id package_id)}; # artificial
				$quotes{$boxID}{$key} = {%quote};
  			}
 			pop @state;
			}, "tagname"],
  	);

	$p->parse($result);
	$p->eof;

	return defined($error) ? $error : \%quotes;
}

1;
